"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const pluralize = require("pluralize");
const change_case = require("change-case");
const fs_1 = require("fs");
class TsBuilder {
    constructor(tables, views) {
        this.tables = tables;
        this.views = views;
        this.mysqlTypes = {
            int: "number",
            decimal: "number",
            double: "number",
            float: "number",
            set: "string",
            bigint: "string",
            longblob: "any",
            tinyint: "boolean",
            date: "Date | string",
            datetime: "Date | string",
            timestamp: "Date | string",
            text: "string",
            longtext: "string",
            mediumtext: "string",
            varchar: "string"
        };
        this.singularizeClassNames = true;
        this.defaultClassName = "export interface";
        this.optionalParameters = true;
    }
    toFilename(name) {
        let filename = this.singularizeClassNames ? pluralize.singular(name) : name;
        return change_case.paramCase(filename) + ".ts";
    }
    static normFolder(folder) {
        if (!folder)
            return "";
        if (!(folder.endsWith("/") || folder.endsWith("\\"))) {
            folder += "/";
        }
        return folder;
    }
    renderFolder(folder, searchString = null) {
        return __awaiter(this, void 0, void 0, function* () {
            folder = TsBuilder.normFolder(folder);
            let tables = this.listTables(searchString);
            let tableClasses = tables.map(t => {
                let filename = this.toFilename(t);
                return {
                    className: this.getClassName(t),
                    filename: filename,
                    fullPath: folder + filename,
                    tableName: t
                };
            });
            tableClasses.forEach(tc => {
                let definition = this.renderTs(tc.tableName, this.tables[tc.tableName]);
                fs_1.writeFileSync(tc.fullPath, definition);
            });
            this.renderTableFile(folder);
            this.renderViewFile(folder);
            // writeFileSync(folder+this.toFilename("inserter", false), new InserterBuilder().renderInserter(tableClasses));
        });
    }
    getTsType(type) {
        var ts = this.mysqlTypes[type];
        if (!ts) {
            console.error("Unknown type " + type);
            ts = "any";
        }
        return ts;
    }
    buildTypeRow(col) {
        let tabs = "\t";
        let optional = this.optionalParameters ? "?" : "";
        let tsType = this.getTsType(col.type);
        let field = col.field;
        return `${tabs} "${field}"${optional}: ${tsType};\n`;
    }
    renderTableFile(folder) {
        folder = TsBuilder.normFolder(folder);
        let start = "export default class Tables { \n";
        let arr = this.listTables().map(t => "\t static " + change_case.constantCase(t) + " = '" + t + "';");
        let content = start + arr.join("\n") + "\n}";
        fs_1.writeFileSync(folder + "tables.ts", content);
    }
    renderViewFile(folder) {
        folder = TsBuilder.normFolder(folder);
        let start = "export default class Views { \n";
        let arr = this.listViews().map(t => "\t static " + change_case.constantCase(t) + " = '" + t + "';");
        let content = start + arr.join("\n") + "\n}";
        fs_1.writeFileSync(folder + "views.ts", content);
    }
    static listTables(schema, searchFor = null) {
        let views = Object.keys(schema);
        if (searchFor)
            views = views.filter(tName => tName.indexOf(searchFor) > -1);
        return views;
    }
    listTables(searchFor = null) {
        return TsBuilder.listTables(this.tables, searchFor);
    }
    listViews(searchFor = null) {
        return TsBuilder.listTables(this.views, searchFor);
    }
    getClassName(tableName) {
        return this.singularizeClassNames ? pluralize.singular(tableName) : tableName;
    }
    renderTs(tableName, table) {
        let className = this.getClassName(tableName);
        var stringBuilder = this.defaultClassName + " " + className + " { \n";
        for (var colName in table) {
            stringBuilder += this.buildTypeRow(table[colName]);
        }
        stringBuilder += "}";
        return stringBuilder;
    }
}
exports.default = TsBuilder;
//# sourceMappingURL=ts-builder.js.map